#!/usr/bin/env python

import os
import sys

import pygame
import pygame.key
import pygame.time
import pygame.mixer
import pygame.surface
import pygame.display

import util
import data

import display

import entity
import weapon

import world
import hud


def init():
    # The mixer's settings
    # The only argument that is not the default
    # is the buffer size, which is reduced to
    # reduce sound latency
    pygame.mixer.pre_init(22050,-16,2,2048)
    
    pygame.init()
    
    pygame.display.set_mode((700,500),pygame.HWSURFACE,0)
    pygame.display.set_caption('Immune Defender - PyWeek 13 (Mutate!)')
    
    util.init()
    
    data.init()
    display.init()
    
    weapon.init()
    entity.init()
    
    world.init()
    hud.init()
    

def run():
    running = True
    state = 'Game'
    
    clock = pygame.time.Clock()
    
    def make_new_world():
        return world.GameWorld()
        
    
    def make_new_hud():
        return hud.Hud()
        
    
    game_world = make_new_world()
    game_hud = make_new_hud()
    
    
    while running:
        clock.tick(40)
        
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
                
                break
                
            
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    running = False
                    
                    break
                    
                
                for ctrl_name,ctrl_key in controlMap.iteritems():
                    if ctrl_key == event.key:
                        inputState[ctrl_name] = True
                        
                    
                
            
            if event.type == pygame.KEYUP:
                for ctrl_name,ctrl_key in controlMap.iteritems():
                    if ctrl_key == event.key:
                        inputState[ctrl_name] = False
                        
                    
                
            
        
        if not running:
            break
            
        
        if state == 'Title':
            display.drawTitleScreen()
            
            # If we have entered the game state during
            # this block...
            if state == 'Game':
                game_world = make_new_world()
                game_hud = make_new_hud()
            
        elif state == 'Game':
            display.drawGameScreen(game_world)
            
            game_world.update()
            
            # If we have entered the title screen state
            # during this block...
            if state == 'Title':
                game_world = None
                game_hud = None
                
            
        
        
    

def main(): init() ; run() ; pygame.quit()

if __name__.lower() == '__main__':
    try:
        import psyco
        psyco.full()
    except ImportError:
        pass
    
    main()
    

