import __builtin__

import pygame
import pygame.rect
import pygame.surface

import util
import data

import weapon
import entity

import random


class GameWorld(object):
    
    def __init__(self):
        super(GameWorld,self).__init__()
        
        self.rect = pygame.rect.Rect((0,0),(2000,450))
        
        self.backdrop = pygame.surface.Surface(self.rect.size,pygame.HWSURFACE)
        
        backdrop_imgs = [data.loadImage('Backdrop-'+str(spam)+'.png') for spam in range(5)]
        
        back_rect = pygame.rect.Rect((0,0),(400,450))
        
        while back_rect.right <= self.backdrop.get_width():
            self.backdrop.blit(random.choice(backdrop_imgs),back_rect.topleft)
            
            back_rect.left += back_rect.width
            
        
        self.player = entity.Player((1000,225))
        
        self.player.world = self
        
        self.baddies = []
        
        self.waveCount = 12
        
        self.projectiles = {
            'Player':[],
            'Baddie':[],
        }
        
        self.explosions = {
            'Damage':[],
            'No Damage':[],
        }
        
        self.spawnWave()
        
    
    def addBaddie(self,new_baddie):
        if (new_baddie not in self.baddies) and new_baddie.alive:
            self.baddies.append(new_baddie)
            
            new_baddie.world = self
            
        
    
    def spawnWave(self):
        
        for spam in range(self.waveCount):
            baddie_type = random.choice([
                'Goblin','Goblin','Goblin','Goblin','Goblin',
                'Bruiser',
                'Bomber','Bomber','Bomber'
            ])
            
            baddie_side = random.choice(['Right','Left'])
            
            baddie_pos = [0,0]
            
            if baddie_side == 'Right':
                baddie_pos[0] = random.randint(
                        (self.player.rect.centerx + 400),
                        (self.player.rect.centerx + 1000))
                baddie_pos[1] = random.randint(50,400)
                
            else:
                baddie_pos[0] = random.randint(
                        (self.player.rect.centerx - 1000),
                        (self.player.rect.centerx - 400))
                baddie_pos[1] = random.randint(50,400)
                
                
            
            self.addBaddie(entity.Baddie(baddie_pos,baddie_type))
            
        
        if self.waveCount >= 24:
            self.waveCount -= random.choice([2,4])
            
        elif self.waveCount >= 22:
            self.waveCount -= random.choice([4,2,2,2,-2])
            
        elif self.waveCount >= 20:
            self.waveCount -= random.choice([2,2,-2])
            
        elif self.waveCount <= 8:
            self.waveCount += random.choice([2,4])
            
        elif self.waveCount <= 10:
            self.waveCount += random.choice([4,2,2,2,-2])
            
        elif self.waveCount <= 12:
            self.waveCount += random.choice([2,2,-2])
            
        else:
            self.waveCount += random.choice([2,4,-2,-4])
            
        
        if self.waveCount < 8:
            self.waveCount = 8
            
        elif self.waveCount > 24:
            self.waveCount = 24
            
        
    
    def addProjectile(self,new_projectile,owner):
        if isinstance(owner,entity.Player):
            if (new_projectile not in self.projectiles['Player']) and new_projectile.alive:
                self.projectiles['Player'].append(new_projectile)
                
                new_projectile.world = self
                
            
        else:
            if (new_projectile not in self.projectiles['Baddie']) and new_projectile.alive:
                self.projectiles['Baddie'].append(new_projectile)
                
                new_projectile.world = self
                
            
        
    
    def addExplosion(self,new_explosion,damage=False):
        if damage:
            if (new_explosion not in self.explosions['Damage']) and new_explosion.alive:
                self.explosions['Damage'].append(new_explosion)
                
                new_explosion.world = self
                
            
        else:
            if (new_explosion not in self.explosions['No Damage']) and new_explosion.alive:
                self.explosions['No Damage'].append(new_explosion)
                
                new_explosion.world = self
                
            
        
    
    def update(self):
        random.random()
        
        self.player.update(self.baddies)
        
        for baddie in self.baddies:
            if abs(baddie.rect.centerx - self.player.rect.centerx) < 800:
                baddie.update()
                
            
            if not baddie.alive:
                self.baddies.remove(baddie)
                
            
        
        
        if len(self.baddies) == 0:
            self.spawnWave()
            
        
        for projectile in self.projectiles['Player']:
            projectile.update(self.baddies)
            
            if projectile.rect.centerx < (self.player.rect.centerx - 400):
                projectile.alive = False
                
            elif projectile.rect.centerx > (self.player.rect.centerx + 400):
                projectile.alive = False
                
            
            if not projectile.alive:
                self.projectiles['Player'].remove(projectile)
            
        for projectile in self.projectiles['Baddie']:
            projectile.update([self.player])
            
            if projectile.rect.centerx < (self.player.rect.centerx - 400):
                projectile.alive = False
                
            elif projectile.rect.centerx > (self.player.rect.centerx + 400):
                projectile.alive = False
                
            
            if not projectile.alive:
                self.projectiles['Player'].remove(projectile)
            
            
        
        #print len(self.explosions['Damage'])
        
        for explosion in self.explosions['Damage']:
            explosion.update(self.baddies)
            
        for explosion in self.explosions['No Damage']:
            explosion.update([])
            
        
    


def init():
    for spam in range(5):
        data.loadImage('Backdrop-'+str(spam)+'.png')
        
    
    # TODO - Pre-load the foreground images
    
    # TODO - Pre-load the "next wave" sound
    
    pass
    

